"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.FileProcessingError = void 0;
exports.addLineNumbers = addLineNumbers;
exports.processPath = processPath;
exports.processPaths = processPaths;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const types_1 = require("./types");
let globalIndex = 1;
/**
 * Custom error class for file processing errors
 */
class FileProcessingError extends Error {
    constructor(message, cause) {
        super(message);
        this.cause = cause;
        this.name = 'FileProcessingError';
    }
}
exports.FileProcessingError = FileProcessingError;
/**
 * Logger class for consistent logging throughout the application
 */
class Logger {
    constructor() {
        this.isDebug = process.env.DEBUG === 'true';
    }
    static getInstance() {
        if (!Logger.instance) {
            Logger.instance = new Logger();
        }
        return Logger.instance;
    }
    debug(message) {
        if (this.isDebug) {
            console.debug(`[DEBUG] ${message}`);
        }
    }
    info(message) {
        console.info(`[INFO] ${message}`);
    }
    error(message, error) {
        console.error(`[ERROR] ${message}`, error || '');
    }
}
exports.Logger = Logger;
/**
 * Checks if a path should be ignored based on patterns
 * @param path - Path to check
 * @param patterns - Patterns to check against
 * @returns boolean indicating whether the path should be ignored
 */
function shouldIgnore(path, patterns) {
    return patterns.some(pattern => {
        const regex = new RegExp(pattern.replace(/\*/g, '.*'));
        return regex.test(path);
    });
}
/**
 * Reads .gitignore rules from a directory
 * @param dirPath - Path to the directory
 * @returns Array of gitignore rules
 */
async function readGitignore(dirPath) {
    const gitignorePath = path.join(dirPath, '.gitignore');
    if (fs.existsSync(gitignorePath)) {
        const content = await fs.promises.readFile(gitignorePath, 'utf-8');
        return content
            .split('\n')
            .map(line => line.trim())
            .filter(line => line && !line.startsWith('#'));
    }
    return [];
}
/**
 * Adds line numbers to the content of a file
 * @param content - The content of the file
 * @returns The content with line numbers added
 */
function addLineNumbers(content) {
    const lines = content.split('\n');
    const padding = String(lines.length).length;
    return lines.map((line, i) => `${String(i + 1).padStart(padding)}  ${line}`).join('\n');
}
/**
 * Prints content in default format
 */
function printDefault(writer, filePath, content, lineNumbers) {
    writer(filePath);
    writer('---');
    if (lineNumbers) {
        content = addLineNumbers(content);
    }
    writer(content);
    writer('');
    writer('---');
}
/**
 * Prints content in XML format
 */
function printAsXml(writer, filePath, content, lineNumbers) {
    writer(`<document index="${globalIndex}">`);
    writer(`<source>${filePath}</source>`);
    writer('<document_content>');
    if (lineNumbers) {
        content = addLineNumbers(content);
    }
    writer(content);
    writer('</document_content>');
    writer('</document>');
    globalIndex++;
}
/**
 * Prints content in Markdown format
 */
function printAsMarkdown(writer, filePath, content, lineNumbers) {
    const ext = path.extname(filePath).slice(1);
    const lang = types_1.EXT_TO_LANG[ext] || '';
    // Figure out how many backticks to use
    let backticks = '```';
    while (content.includes(backticks)) {
        backticks += '`';
    }
    writer(filePath);
    writer(`${backticks}${lang}`);
    if (lineNumbers) {
        content = addLineNumbers(content);
    }
    writer(content);
    writer(backticks);
}
/**
 * Processes a path and returns its content
 */
async function processPath(filePath, options, writer) {
    const logger = Logger.getInstance();
    try {
        const stats = await fs.promises.stat(filePath);
        if (stats.isFile()) {
            try {
                const content = await fs.promises.readFile(filePath, 'utf-8');
                if (options.claudeXml === true) {
                    printAsXml(writer, filePath, content, options.lineNumbers === true);
                }
                else if (options.markdown === true) {
                    printAsMarkdown(writer, filePath, content, options.lineNumbers === true);
                }
                else {
                    printDefault(writer, filePath, content, options.lineNumbers === true);
                }
            }
            catch (error) {
                if (error instanceof Error && error.message.includes('ENOENT')) {
                    logger.error(`Warning: Skipping file ${filePath} due to UnicodeDecodeError`);
                }
                else {
                    throw error;
                }
            }
        }
        else if (stats.isDirectory()) {
            const entries = await fs.promises.readdir(filePath, { withFileTypes: true });
            // Filter hidden files/directories
            let filteredEntries = entries;
            if (!options.includeHidden) {
                filteredEntries = entries.filter(entry => !entry.name.startsWith('.'));
            }
            // Read gitignore rules if not ignored
            let gitignoreRules = [];
            if (!options.ignoreGitignore) {
                gitignoreRules = await readGitignore(filePath);
            }
            // Filter based on gitignore rules
            if (!options.ignoreGitignore) {
                filteredEntries = filteredEntries.filter(entry => {
                    const fullPath = path.join(filePath, entry.name);
                    return !shouldIgnore(fullPath, gitignoreRules);
                });
            }
            // Filter based on ignore patterns
            if (options.ignorePatterns && options.ignorePatterns.length > 0) {
                filteredEntries = filteredEntries.filter(entry => {
                    const fullPath = path.join(filePath, entry.name);
                    if (!options.ignoreFilesOnly || !entry.isDirectory()) {
                        return !shouldIgnore(fullPath, options.ignorePatterns || []);
                    }
                    return true;
                });
            }
            // Filter based on extensions
            if (options.extensions && options.extensions.length > 0) {
                filteredEntries = filteredEntries.filter(entry => {
                    if (entry.isFile()) {
                        return options.extensions?.some(ext => entry.name.endsWith(ext)) || false;
                    }
                    return true;
                });
            }
            // Process files and directories
            for (const entry of filteredEntries.sort()) {
                const fullPath = path.join(filePath, entry.name);
                if (entry.isDirectory()) {
                    await processPath(fullPath, options, writer);
                }
                else {
                    try {
                        const content = await fs.promises.readFile(fullPath, 'utf-8');
                        if (options.claudeXml === true) {
                            printAsXml(writer, fullPath, content, options.lineNumbers === true);
                        }
                        else if (options.markdown === true) {
                            printAsMarkdown(writer, fullPath, content, options.lineNumbers === true);
                        }
                        else {
                            printDefault(writer, fullPath, content, options.lineNumbers === true);
                        }
                    }
                    catch (error) {
                        if (error instanceof Error && error.message.includes('ENOENT')) {
                            logger.error(`Warning: Skipping file ${fullPath} due to UnicodeDecodeError`);
                        }
                        else {
                            throw error;
                        }
                    }
                }
            }
        }
    }
    catch (error) {
        throw new FileProcessingError(`Failed to process path: ${filePath}`, error);
    }
}
/**
 * Processes multiple paths and returns the combined output
 */
async function processPaths(paths, options) {
    const output = [];
    const writer = (content) => output.push(content);
    for (const path of paths) {
        await processPath(path, options, writer);
    }
    return output.join('\n');
}
