/**
 * Configuration options for the file processing
 */
export interface Config {
    /** Directory to process files from */
    dir: string;
    /** File extensions to include (e.g., ['.js', '.ts']) */
    extensions?: string[];
    /** Whether to include hidden files */
    includeHidden?: boolean;
    /** Whether to include node_modules directory */
    includeNodeModules: boolean;
    /** Whether to include .git directory */
    includeGit: boolean;
    /** Whether to include .vscode directory */
    includeVscode: boolean;
    /** Whether to include .idea directory */
    includeIdea: boolean;
    /** Whether to include .DS_Store files */
    includeDsStore: boolean;
    /** Whether to include .gitignore files */
    includeGitignore: boolean;
    /** Whether to include .env files */
    includeEnv: boolean;
    /** Whether to include .env.local files */
    includeEnvLocal: boolean;
    /** Whether to include .env.development files */
    includeEnvDevelopment: boolean;
    /** Whether to include .env.test files */
    includeEnvTest: boolean;
    /** Whether to include .env.production files */
    includeEnvProduction: boolean;
    /** Whether to include .env.development.local files */
    includeEnvDevelopmentLocal: boolean;
    /** Whether to include .env.test.local files */
    includeEnvTestLocal: boolean;
    /** Whether to include .env.production.local files */
    includeEnvProductionLocal: boolean;
    /** Whether to only process files specified in ignore patterns */
    ignoreFilesOnly?: boolean;
    /** Whether to ignore .gitignore rules */
    ignoreGitignore?: boolean;
    /** Custom ignore patterns to apply */
    ignorePatterns?: string[];
}
/**
 * Represents a file in the project
 */
export interface File {
    /** Relative path of the file */
    path: string;
    /** Content of the file */
    content: string;
}
/**
 * Represents a directory in the project
 */
export interface Directory {
    /** Relative path of the directory */
    path: string;
    /** List of files in the directory */
    files: File[];
    /** List of subdirectories */
    subdirectories: Directory[];
}
/**
 * Represents the entire project structure
 */
export interface Project {
    /** Root directory of the project */
    root: Directory;
    /** Configuration used to generate the project structure */
    config: Config;
}
/**
 * File extension to language mapping
 */
export declare const EXT_TO_LANG: {
    [key: string]: string;
};
/**
 * Options for processing files
 */
export interface Options {
    /**
     * File extensions to include
     */
    extensions?: string[];
    /**
     * Whether to include hidden files and directories
     */
    includeHidden: boolean;
    /**
     * Whether to ignore files only (not directories) when using ignore patterns
     */
    ignoreFilesOnly: boolean;
    /**
     * Whether to ignore .gitignore files
     */
    ignoreGitignore: boolean;
    /**
     * Patterns to ignore
     */
    ignorePatterns?: string[];
    /**
     * Output file path
     */
    outputFile?: string;
    /**
     * Whether to output in Claude XML format
     */
    claudeXml: boolean;
    /**
     * Whether to output in Markdown format
     */
    markdown: boolean;
    /**
     * Whether to add line numbers
     */
    lineNumbers: boolean;
    /**
     * Whether to use NUL character as separator when reading from stdin
     */
    useNullSeparator: boolean;
}
/**
 * Represents the content of a processed file
 */
export interface FileContent {
    /** Relative path of the file */
    path: string;
    /** Content of the file */
    content: string;
}
